# -*- coding: utf-8 -*-
try:
    from tqdm import tqdm

    tqdm_ = tqdm(unit="B", unit_scale=True, maxinterval=0)
    # setting maxinterval=0 to disable tqdm's internal monitor
except:
    tqdm_ = None


def report(current, max):
    if tqdm is not None:
        tqdm_.total = max
        tqdm_.n = current
        tqdm_.refresh()


def close():
    # silence tqdm
    if tqdm is not None:
        tqdm_.disable = True



# -*- coding: utf-8 -*-
import os
from sys import path
from bilibili_toolman.providers import DownloadResult

__desc__ = "本地文件"
__cfg_help__ = """
    cover (str) - 封面图片路径
e.g. --localfile "le videos/" --opts cover="le cover.png" --tags ..."""
options = {"cover": ""}


def update_config(opt):
    global options
    options = {**options, **opt}


def download_video(res) -> DownloadResult:
    results = DownloadResult()
    if (
        not os.path.isfile(res)
        and not os.path.isdir(res)
        or (options["cover"] and os.path.isfile(options["cover"]))
    ):
        raise FileNotFoundError("%s - file not found" % res)

    def append(res):
        with DownloadResult() as result:
            result.video_path = res
            result.cover_path = options["cover"]
            result.title = os.path.basename(res)
            result.soruce = "bilibili-toolman"
            result.description = "[automated upload of file %s]" % res
        results.results.append(result)

    if os.path.isfile(res):
        append(res)
    else:
        for f in os.listdir(res):
            append(os.path.join(res, f))
    results.title = os.path.basename(res)
    results.soruce = "bilibili-toolman"
    results.description = "[automated upload of file %s]" % res
    return results



# -*- coding: utf-8 -*-
"""bilibili - PC API implementation"""
from json import loads
from functools import wraps
from time import time
from requests.sessions import Session
from typing import Iterable, Tuple, Union
from urllib.parse import quote, urlencode
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5
from hashlib import md5
from base64 import b64encode
import math, logging

from bilibili_toolman.bilisession.web import BiliSession as BiliWebSession
from bilibili_toolman.bilisession.common import (
    FileIterator,
    JSONResponse,
    LoginException,
    ReprExDict,
    file_manager,
    check_file,
)
from bilibili_toolman.bilisession.common.submission import Submission

logger = logging.getLogger("ClientSession")

class RecaptchaRequiredException(Exception):
    def __init__(self, recaptcha_url) -> None:
        self.recaptcha_url = recaptcha_url
        super().__init__(recaptcha_url)        

def PCOnlyAPI(_classmethod):
    @wraps(_classmethod)
    def wrapper(self, *a, **k):
        assert type(self) == BiliSession, "限 PC API 使用"
        return _classmethod(self, *a, **k)
    return wrapper

class Crypto:
    """THANK YOU! https://github.com/FortuneDayssss/BilibiliUploader"""

    APPKEY = "661dbf0ee792f083"
    APPSECRET = "397c810e126bfbbb584667583e11976a"
    # thx x69dbg ( ͡° ͜ʖ ͡°)
    @staticmethod
    def iterable_md5(stream: Iterable) -> str:
        md5_ = md5()
        for chunk in stream:
            md5_.update(chunk)
        return md5_.hexdigest()

    @staticmethod
    def md5(data: Union[str, bytes]) -> str:
        """generates md5 hex dump of `str` or `bytes`"""
        if type(data) == str:
            return md5(data.encode()).hexdigest()
        return md5(data).hexdigest()

    @staticmethod
    def rsa_pkcs115_encrypt_as_base64(content: bytes, pkcs115_pubkey: str) -> str:
        key = RSA.import_key(pkcs115_pubkey)
        cipher = PKCS1_v1_5.new(key)
        return b64encode(cipher.encrypt(content))

    @staticmethod
    def encrypt_login_password(password: str, oauth2_getkey_json: dict) -> str:
        _hash, key = (
            oauth2_getkey_json["data"]["hash"],
            oauth2_getkey_json["data"]["key"],
        )
        return Crypto.rsa_pkcs115_encrypt_as_base64(
            (_hash + password).encode("utf-8"), key
        )

    @staticmethod
    def sign(data: Union[str, dict]) -> str:
        """salted sign funtion for `dict`(converts to qs then parse) & `str`"""
        if isinstance(data, dict):
            _str = urlencode(data)
        elif type(data) != str:
            raise TypeError
        return Crypto.md5(_str + Crypto.APPSECRET)

class SignedDict(dict):
    @property
    def sorted(self):
        """returns a alphabetically sorted version of `self`"""
        return dict(sorted(self.items()))

    @property
    def signed(self):
        """returns our sorted self with calculated `sign` as a new key-value pair at the end"""
        _sorted = self.sorted
        return {**_sorted, "sign": Crypto.sign(_sorted)}

class ClientUploadChunk(FileIterator):
    url_endpoint: str
    params: dict
    headers: dict
    files: dict
    cookies: dict
    session: Session

    def upload_via_session(self, session=None):
        chunk_bytes = self.to_bytes()
        md5 = Crypto.md5(chunk_bytes)
        for retries in range(1, BiliSession.RETRIES_UPLOAD_ID + 1):
            try:
                resp = (session or self.session).post(
                    self.url_endpoint,
                    params=self.params,
                    headers=self.headers,
                    files={
                        **self.files,
                        "md5": (None, md5),
                        "file": (self.path, chunk_bytes, "application/octet-stream"),
                    },
                    cookies=self.cookies,
                )
                assert resp.json()["OK"] == 1, resp.text
                return True
            except Exception as e:
                logger.warning("第 %s 次重试时：%s" % (retries, e))
        return False

class BiliSession(BiliWebSession):
    """哔哩哔哩上传助手 API"""

    TYPE = "client"

    UPLOAD_CHUNK_SIZE = 2 * (1 << 20)
    BUILD_VER = (2, 3, 0, 1073)
    BUILD_NO = int(        
        BUILD_VER[0] * 1e6 + BUILD_VER[1] * 1e5 + BUILD_VER[2] * 1e4 + BUILD_VER[3]
    )
    BUILD_STR = ".".join(map(lambda v: str(v), BUILD_VER))

    UPLOAD_PROFILE = "ugcfr/pc3"

    DEFAULT_UA = ""

    RETRIES_UPLOAD_ID = 5

    MISC_MAX_TITLE_LENGTH = 80
    MISC_MAX_DESCRIPTION_LENGTH = 1500  # somehow larger than expected

    def __init__(self) -> None:
        Session.__init__(self)  # no need to init the web variant
        self.headers = {
            "User-Agent": BiliSession.DEFAULT_UA,
            "Accept-Encoding": "gzip,deflate",
        }
        self.login_tokens = dict()        
        self.logger = logger
        
    # region Properties
    @property
    def mid(self):
        return self.login_tokens["mid"]

    @property
    def access_token(self):
        return self.login_tokens["access_token"]

    @property
    def access_key_param(self):
        """Singed dictionary containing only `access_key` key-value pair"""
        return SignedDict({"access_key": self.access_token,"build" : self.BUILD_NO}).signed

    # endregion

    # region Overrides
    def _preupload(self):
        return self.get(
            "http://member.bilibili.com/preupload",
            params={
                "access_key": self.access_token,
                "mid": self.mid,
                "profile": self.UPLOAD_PROFILE,
            },
        )

    def _upload_cover(self, image_binary: bytes, image_mime: str):
        return self.post(
            "https://member.bilibili.com/x/vu/client/cover/up",
            params=self.access_key_param,
            files={"file": ("cover.png", image_binary, image_mime)},
        )

    def _submit_submission(self, submission: Submission):
        return self.post(
            "https://member.bilibili.com/x/vu/client/add",
            params=self.access_key_param,
            json={"build": self.BUILD_VER[-1], **submission.archive},
        )

    def _list_archives(self, params):
        return self.get(
            "https://member.bilibili.com/x/client/archives",
            params=SignedDict({"access_key": self.access_token, **params}).signed,
        )

    @JSONResponse
    def _view_archive(self, bvid):
        return self.get(
            "https://member.bilibili.com/x/client/archive/view",
            params=SignedDict(
                {
                    "access_key": self.access_token,
                    "bvid": bvid,
                    "build": self.BUILD_VER[-1],
                }
            ).signed,
        )

    def _edit_archive(self, json: dict):
        return self.post(
            "https://member.bilibili.com/x/vu/client/edit",
            params=self.access_key_param,
            json=SignedDict(
                {
                    "build": self.BUILD_VER[-1],
                    "no_reprint": 0,
                    "open_elec": 0,
                    **json,
                }
            ).sorted,
        )

    def _delete_archive(self, bvid):
        return self.post(
            "http://member.bilibili.com/x/client/archive/delete",
            params=self.access_key_param,
            data={"bvid": bvid},
        )

    # endregion

    # region Client-specific APIs
    @JSONResponse
    def _oauth2_getkey(self):
        return self.get(
            "https://passport.bilibili.com/x/passport-login/web/key",
            params=SignedDict({"appkey": Crypto.APPKEY, "platform": "pc"}).signed,
        )

    def _post_complete_upload(self, complete_url, size, basename, md5, chunkcount):
        return self.post(
            complete_url,
            data={
                "chunks": chunkcount,
                "filesize": size,
                "md5": md5,
                "name": basename,
                "version": self.BUILD_STR,
            },
        )

    @PCOnlyAPI
    @JSONResponse
    def DeleteArchive(self, bvid):
        """删除作品

        Args:
            bvid
        """
        return self._delete_archive(bvid)

    @PCOnlyAPI
    @JSONResponse
    def LoginViaUsername(self, username: str, password: str):
        """用户名密码登陆

        Args:
            username (str), password (str)

        Returns:
            dict
        """
        raise DeprecationWarning(
            "Deprecated by %s. Use SMS login instead." % self.BUILD_STR
        )
        oauth_json = self._oauth2_getkey()
        resp = self.post(
            "https://passport.bilibili.com/x/passport-login/oauth2/login",
            data=SingableDict(
                {
                    "appkey": Crypto.APPKEY,
                    "platform": "pc",
                    "password": Crypto.encrypt_login_password(password, oauth_json),
                    "username": quote(username),
                    "ts": int(time() * 1000),
                    "device_name": "",
                    "device_id": "",
                    "buvid": "",
                }
            ).signed,
        )
        try:
            self.login_tokens.update(resp.json()["data"]["token_info"])
        except Exception as e:
            raise LoginException(resp, e)
        return resp

    GEETEST_HOOK = '''eval("Object._assign=Object.assign;Object.assign=(...args)=>{if(args[1]['geetest_challenge']){document.body.innerHTML=JSON.stringify(args[1]);}return Object._assign(...args)}")'''
    def ManuallySolveGeetestRecaptcha(self,recaptcha_url):
        from urllib.parse import parse_qsl
        recaptcha = dict(parse_qsl(recaptcha_url.split('?')[-1]))
        logger.warning("完成极验步骤须知：")      
        logger.warning("1. 浏览器中打开所示验证链接")
        logger.info("%s",recaptcha_url)
        logger.warning("2. 在浏览器地址栏输入 javascript: 并粘贴以下内容并回车")
        logger.info("%s",self.GEETEST_HOOK)
        logger.warning("3. 正确完成验证后，复制出现的 JSON")
        logger.warning("4. 粘贴于此处并回车：")
        gee_validation = loads(input())                
        self.login_tokens["gee_captcha"] = {
            "recaptcha_token" : recaptcha["recaptcha_token"],
            "gee_challenge":gee_validation["geetest_challenge"],
            "gee_validate":gee_validation["geetest_validate"],
            "gee_seccode":gee_validation["geetest_seccode"]
        }
        return self.login_tokens["gee_captcha"]

    @PCOnlyAPI
    @JSONResponse
    def RenewSMSCaptcha(self, tel: str, cid=86):
        """发送验证码

        Args:
            tel (str) : 手机号
            cid (int) : 国家代码

        Returns:
            dict
        """
        resp = self.post(
            "https://passport.bilibili.com/x/passport-login/sms/send",
            data=SignedDict(
                {
                    "appkey": Crypto.APPKEY,
                    "buvid": "",
                    "cid": str(cid),
                    "device_id": "",
                    "device_name": "",
                    "device_platform" : "",
                    "platform": "pc",
                    "tel": str(tel),
                    "ts": int(time()),
                    **(
                        self.login_tokens["gee_captcha"] if "gee_captcha" in self.login_tokens else  {}
                    ),
                }
            ).signed
        )
        try:
            assert resp.json()["data"]["recaptcha_url"] == ""
            self.login_tokens["captcha_key"] = resp.json()["data"]["captcha_key"]
        except Exception as e:
            data = resp.json()["data"]
            if data and "recaptcha_url" in data:
                logger.warning("需要人机交互完成校验")
                raise RecaptchaRequiredException(data["recaptcha_url"])
            raise LoginException(resp, e)
        return resp

    @PCOnlyAPI
    @JSONResponse
    def LoginViaSMSCaptcha(self, tel: str, code: int, cid=86):
        """验证码登陆

        Args:
            tel (str) : 手机号
            code (str) : 验证码
            cid (int) : 国家代码

        Returns:
            dict
        """
        assert self.login_tokens.get(
            "captcha_key", None
        ), "`RenewSMSCaptcha` not called or failed."
        resp = self.post(
            "https://passport.bilibili.com/x/passport-login/login/sms",
            data=SignedDict(
                {
                    "appkey": Crypto.APPKEY,
                    "platform": "pc",
                    "captcha_key": self.login_tokens["captcha_key"],
                    "code": str(code),
                    "tel": str(tel),
                    "cid": str(cid),
                    "ts": int(time() * 1000),
                    "device_name": "",
                    "device_id": "",
                    "buvid": "",
                }
            ).signed,
        )
        try:
            self.login_tokens.update(resp.json()["data"]["token_info"])
        except Exception as e:
            raise LoginException(resp, e)
        return resp

    def UploadVideo(self, path: str) -> Tuple[str, None]:
        """上传视频

        Args:
            path (str): 视频文件路径

        Returns:
            Tuple[str,None]: [远端 URI,None]
        """
        path, basename, size = check_file(path)
        preupload_token = self._preupload().json()
        # preprae the chunks then uploads them
        chunksize = self.UPLOAD_CHUNK_SIZE
        chunkcount = math.ceil(size / chunksize)
        file_manager.open(path)
        logger.debug("上传分块: %s" % chunkcount)
        logger.debug("分块大小: %s B" % chunksize)

        def iter_chunks():
            for chunk_n in range(0, chunkcount):
                start = chunksize * chunk_n
                chunk = ClientUploadChunk(path, start, min(start + chunksize, size))
                chunk.session = self
                chunk.url_endpoint = preupload_token["url"]
                chunk.files = {
                    "version": (None, self.BUILD_STR),
                    "filesize": (None, chunksize),
                    "chunk": (None, chunk_n),
                    "chunks": (None, chunkcount),
                }
                chunk.cookies = {"PHPSESSID": preupload_token["filename"]}
                yield chunk

        self._upload_chunks_to_endpoint_blocking(iter_chunks())
        # recalulating md5
        md5_ = Crypto.iterable_md5(FileIterator(path, 0, size))
        file_manager.close(path)
        logger.debug("MD5: %s" % md5_)
        # finalizing upload
        post_r = self._post_complete_upload(
            preupload_token["complete"], size, basename, md5_, chunkcount
        )
        logger.info("远端结点： %s" % preupload_token.get("filename", "<failed>"))
        logger.debug("上传完毕： %s" % ReprExDict(post_r.json()))
        return preupload_token["filename"], None

    # endregion

    # region Pickling
    def __dict__(self):
        return {
            "cookies": self.cookies,
            "login_tokens": self.login_tokens,
            "session": self.TYPE,
        }

    def update(self, state_dict: dict):
        self.cookies = state_dict["cookies"]
        self.login_tokens = state_dict["login_tokens"]

# endregion



# -*- coding: utf-8 -*-
from bilibili_toolman.bilisession.web import BiliSession
from bilibili_toolman.bilisession.client import RecaptchaRequiredException
from bilibili_toolman.bilisession.common import LoginException
from bilibili_toolman.bilisession.common.submission import Submission
from bilibili_toolman.providers import DownloadResult
from bilibili_toolman.cli import (
    local_args as largs,
    setup_logging,
    prepare_temp,
    prase_args,
    sanitize,
    truncate,
    AttribuitedDict,
)

from collections import defaultdict
import logging, sys, urllib.parse

TEMP_PATH = "temp"

sess_upload: BiliSession
sess_submit: BiliSession
logger = logging.getLogger("toolman")


def download_sources(provider, arg) -> DownloadResult:
    resource = arg.resource
    opts = arg.opts
    try:
        opts = urllib.parse.parse_qs(opts, keep_blank_values=False)
        provider.update_config({k: v[-1].replace(";", "") for k, v in opts.items()})
    except:
        opts = "无效选项"
    """Passing options"""
    logger.info("下载源视频")
    logger.info("  - Type: %s - %s" % (provider.__name__, provider.__desc__))
    logger.info("  - URI : %s" % resource)
    """Downloading source"""
    try:
        return provider.download_video(resource)
    except Exception as e:
        logger.error("无法下载指定资源 - %s" % e)
        return


def upload_sources(sources: DownloadResult, arg):
    """To perform a indivudial task

    If multiple videos are given by the provider,the submission will be in multi-parts (P)
    Otherwise,only the given video is uploaded as a single part subject

    Args:

        provider - one of the modules of `providers`
        arg - task arguments dictionary
            * resource - resoucre URI (must have)
            - opts     - options for uploader in query string e.g. format=best
            - See `utils.local_args` for more arguments,along with thier details
    """
    submission = Submission()
    if not sources:
        return None, True
    logger.info("上传资源数：%s" % len(sources.results))

    def format(source):
        blocks = defaultdict(
            lambda: "...",
            {"title": source.title, "desc": source.description, **source.extra},
        )
        title = truncate(
            sanitize(arg.title.format_map(blocks)), sess_submit.MISC_MAX_TITLE_LENGTH
        )
        description = truncate(
            sanitize(arg.desc.format_map(blocks)),
            sess_submit.MISC_MAX_DESCRIPTION_LENGTH,
        )
        return blocks, title, description

    for source in sources.results:
        """If one or multipule sources"""
        blocks, title, description = format(source)
        logger.info("准备上传: %s" % title)
        """Summary trimming"""
        endpoint = None
        for _ in range(0, sess_upload.RETRIES_UPLOAD_ID):
            try:
                endpoint, bid = sess_upload.UploadVideo(source.video_path)
                cover_url = (
                    sess_upload.UploadCover(source.cover_path)["data"]["url"]
                    if source.cover_path
                    else ""
                )
                break
            except Exception as e:
                logger.warning("%s 上传失败! - %s - 重试" % (source, e))
        if not endpoint:
            logger.error("URI 获取失败 - 跳过")
            continue
        logger.info("资源已上传")
        from bilibili_toolman.cli import precentage_progress

        precentage_progress.close()
        with Submission() as video:
            """Creatating a video per submission"""
            video.cover_url = cover_url
            video.video_endpoint = endpoint
            video.biz_id = bid
            """Sources identifiers"""
            video.copyright = (
                submission.COPYRIGHT_REUPLOAD
                if not arg.original
                else submission.COPYRIGHT_ORIGINAL
            )
            video.source = sources.soruce
            """Thread,Tags,Description & Title"""
            video.thread = arg.thread_id
            video.tags = arg.tags.format_map(blocks).split(",")
            video.description = description  # Description per-part is implemented. Dunno why their frontend decided to discard this data.
            # Still shows up in responses though. Might be of use sometime.
            video.title = title  # This shows up as title per-part
        """Use the last given thread per multiple uploads,while the tags are extended."""
        submission.thread = video.thread or submission.thread
        submission.tags.extend(video.tags)
        submission.videos.append(video)  # to the main submission
    """Filling submission info"""
    submission.copyright = (
        submission.COPYRIGHT_REUPLOAD
        if not arg.original
        else submission.COPYRIGHT_ORIGINAL
    )
    submission.no_reprint = (
        submission.REPRINT_ALLOWED
        if not arg.no_reprint
        else submission.REPRINT_DISALLOWED
    )
    """Use title & description from main sources (e.g. Uploading a YT Playlist)"""
    blocks, title, description = format(sources)
    submission.title = title
    submission.description = description  # This is the only description that gets shown
    submission.source = sources.soruce
    """Upload cover images for all our submissions as well"""
    cover_url = (
        sess_upload.UploadCover(sources.cover_path)["data"]["url"]
        if sources.cover_path
        else ""
    )
    submission.cover_url = cover_url
    """Finally submitting the video"""
    if not arg.no_submit:
        submit_result = sess_submit.SubmitSubmission(
            submission, seperate_parts=arg.seperate_parts
        )
        dirty = False
        for result in submit_result["results"]:
            if result["code"] == 0:
                logger.info("上传成功 - BVid: %s" % result["data"]["bvid"])
            else:
                logger.warning("%s 上传失败 : %s" % (submission, result["message"]))
                dirty = True
        return submit_result, dirty
    else:
        logger.warning("已跳过稿件提交")
        return "", False


global_args, local_args = None, None


def setup_session():
    """Setup session with cookies in query strings & setup temp root"""
    global sess_upload
    global sess_submit

    def setup_params(sess):
        if global_args.http:
            logger.warning("强制使用 HTTP")
            sess.FORCE_HTTP = True
        if global_args.noenv:
            logger.warning("不使用环境变量；请求将绕过代理")
            sess.trust_env = False

    if global_args.cookies:
        from bilibili_toolman.bilisession.web import BiliSession

        sess = BiliSession(global_args.cookies)
        setup_params(sess)
        user = sess.Self
        if not "uname" in user["data"]:
            logger.error("Cookies无效: %s" % user["message"])
            return False
        logger.warning("Web端 API 需 Lv3+ 及 1000+ 关注量才可多 P 上传，若出错请启用 --seperate_parts")
        sess_upload = sess
        sess_submit = sess
    elif global_args.sms:
        from bilibili_toolman.bilisession.client import BiliSession

        sess = BiliSession()
        setup_params(sess)
        logger.warning("短信验证码有日发送条数限制（5 条），无论验证成败，超限后将无法发送验证码")
        logger.warning("建议使用 --save 保存验证凭据，再次使用则利用 --load 读取，而不需再次登陆")
        logger.info("准备登陆，输入手机号后，按Enter发送验证码")
        phone = input()
        try:
            ret = sess.RenewSMSCaptcha(phone)
        except RecaptchaRequiredException as r:
            logger.warning("准备人工校验，若出错请重启运行")
            recaptcha_url = r.recaptcha_url
            sess.ManuallySolveGeetestRecaptcha(recaptcha_url)
            ret = sess.RenewSMSCaptcha(phone)
        assert ret['code'] == 0,"发送失败 ：%s" % ret
        logger.debug("验证码已发送")
        for i in range(0, 5):
            logger.info("输入验证码:")
            captcha = input()
            try:
                sess.LoginViaSMSCaptcha(phone, captcha)
                sess_upload = sess
                sess_submit = sess
                break
            except LoginException as e:
                logger.error("验证码无效，请重试：%s" % e)
        if i == 4:
            logger.critical("多次重试后无法登录")
            return False
    elif global_args.load:
        from bilibili_toolman.bilisession.web import BiliSession
        sess = BiliSession.from_base64_string(global_args.load)
        setup_params(sess)
        sess_upload = sess
        sess_submit = sess
    else:
        logger.error("未提供凭据")
        return False
    # Overriding credentials
    if global_args.load_upload:
        sess = BiliSession.from_base64_string(global_args.load_upload)
        setup_params(sess)
        sess_upload = sess
    if global_args.load_submit:
        sess = BiliSession.from_base64_string(global_args.load_submit)
        setup_params(sess)
        sess_submit = sess
    return True


def __main__():
    global global_args, local_args
    global sess_submit, sess_upload
    setup_logging()
    args = prase_args(sys.argv)
    if args:
        global_args, local_args = args
    else:
        sys.exit(1)
    """Parsing args"""
    if not setup_session():
        logger.fatal("登陆失败！")
        sys.exit(2)

    if global_args.save and not (global_args.load_upload or global_args.load_submit):
        logger.info("保存登录凭据")
        print(sess_submit.to_base64_string())
        sys.exit(0)

    for desc, sess in [("提交用", sess_submit), ("上传用", sess_upload)]:
        logger.warning("%s配置：" % desc)
        if sess.TYPE == "web":  # using Web APIs
            bup = {"ws", "qn", "bda2"}
            bupfetch = {
                "kodo",
                "gcs",
                "bos",
            }  # TODO : Actually implementing bupfetch routes
            if global_args.cdn in bup:
                sess.UPLOAD_PROFILE = "ugcupos/bup"
            elif global_args.cdn in bupfetch:
                sess.UPLOAD_PROFILE = "ugcupos/bupfetch"
            sess.UPLOAD_CDN = global_args.cdn
            logger.info("Web 端 API @ ID:%s" % sess.Self["data"]["uname"])
            logger.debug("CDN ： %s [%s]" % (sess.UPLOAD_CDN, sess.UPLOAD_PROFILE))
        elif sess.TYPE == "client":  # using client APIs
            logger.info("上传助手 API @ MID:%s" % sess.mid)

    prepare_temp(TEMP_PATH)
    # Output current settings
    logger.info("任务总数: %s" % len(local_args))
    success, failure = [], []
    fmt = lambda s: ("×", "√")[s] if type(s) is bool else s
    for provider, arg_ in local_args:
        arg = AttribuitedDict(arg_)
        logger.info("任务信息：")
        for k, v in largs.items():
            logger.info("  - %s : %s" % (list(v.values())[0].split()[0], fmt(arg[k])))
        sources = download_sources(provider, arg)
        if arg.no_upload:
            logger.warn("已跳过上传")
        else:
            result, dirty = upload_sources(sources, arg)
            if not dirty:
                success.append((arg, result))
            else:
                failure.append((arg, None))

    if not failure:
        logger.info("任务完毕")
        sys.exit(0)
    logger.warning("上传未完毕：检查标签，标题，描述内有无非法字符或超长字串，或尝试重新登陆；如遇其他问题请提交issue")
    sys.exit(1)



# -*- coding: utf-8 -*-
"""Content provider modules"""
from typing import List


class DownloadResult:
    video_path: str = ""
    cover_path: str = ""

    title: str = ""
    description: str = ""
    soruce: str = ""

    results = None

    extra = dict()

    @property
    def results(self):
        """A list of sub download results"""
        self._results: List[DownloadResult]  # hack to allow typing of ourself
        return self._results

    def __init__(self) -> None:
        self._results = []

    def __enter__(self):
        return DownloadResult()

    def __exit__(self, *args):
        pass

    def __repr__(self) -> str:
        return "< title : %s , src : %s>" % (self.title, self.soruce)


from bilibili_toolman.providers import youtube as provider_youtube
from bilibili_toolman.providers import localfile as provider_localfile



# -*- coding: utf-8 -*-
import setuptools, bilibili_toolman

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="bilibili-toolman",
    version=bilibili_toolman.__version__,
    author="greats3an",
    author_email="greats3an@gmail.com",
    description=bilibili_toolman.__desc__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/greats3an/bilibili-toolman",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=["requests", "pycryptodome", "coloredlogs", "yt-dlp", "tqdm"],
    entry_points={
        "console_scripts": ["bilibili-toolman=bilibili_toolman.cli.main:__main__"]
    },
    python_requires=">=3.6",
)
